/* 
* -cr_beg-
* ********************************************************************
* Licensed Materials - Property of IBM
* Restricted Materials of IBM
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
*  Copyright IBM Corp. 2013
* ********************************************************************
* -cr_end-
*/ 
#ifdef WIN32
#define EXPORT __declspec(dllexport)
#else
#define EXPORT
#endif

#define ODPP_TERA_MAX_FUNC_NAME_CHARS  30
#define ODPP_TERA_MAX_SPEC_NAME_CHARS  30
#define ODPP_TERA_MAX_ERROR_MSG_CHARS  256

#define ODPPDATATYPE_TERA_UDT          -1

typedef struct ODPPTeraMaskSQLCA
{
   int iErrCode;
   char *sqlstate;
   short charSetId;
   unsigned char* functionName;
   unsigned char* specificName;
   unsigned char* errorMessage;
#ifdef WIN32
   unsigned short* functionNameW;
   unsigned short* specificNameW;
   unsigned short* errorMessageW;
#else
   unsigned int functionNameW[ODPP_TERA_MAX_FUNC_NAME_CHARS + 1];
   unsigned int specificNameW[ODPP_TERA_MAX_SPEC_NAME_CHARS + 1];
   unsigned int errorMessageW[ODPP_TERA_MAX_ERROR_MSG_CHARS + 1];
   unsigned short* errorMessageU;
#endif
   void (*pFNC_GetStructuredAttributeCount)  (UDT_HANDLE udtHandle,
                                              int *Attribute_count);
   void (*pFNC_GetStructuredAttributeByNdx)  (UDT_HANDLE udtHandle,
                                              int attributeIndex,
                                              void* returnValue,
                                              int bufSize,
                                              int* nullIndicator,
                                              int* length);
   void (*pFNC_SetStructuredAttributeByNdx)  (UDT_HANDLE udtHandle,
                                              int attributeIndex,
                                              void* newValue,
                                              int nullIndicator,
                                              int length);
   void (*pFNC_GetStructuredAttributeInfo)   (UDT_HANDLE AnyStructuredUdt,
                                              int attribute_position,
                                              int bufSize,
                                              attribute_info_t* attributeArray);
   FNC_Mode (*pFNC_GetPhase)                 (FNC_Phase* Phase);
   void* (*pFNC_TblAllocCtx)                 (int length);
   void* (*pFNC_TblGetCtx)                   (void);
   FNC_ColumnDef_t* (*pFNC_TblGetColDef)     (void);
   void* (*pFNC_malloc)                      (size_t size);
   void (*pFNC_free)                         (void *ptr);
   void (*pFNC_Trace_String)                 (void *TraceStr);
   void (*pFNC_Trace_Write_DL)               (int argc, void *argv[], int length[]);
} ODPPTeraMaskSQLCA;

typedef struct ODPPTeraTime
{
   int seconds; /* microseconds */
   signed char hour;
   signed char minute;
} ODPPTeraTime;

typedef struct ODPPTeraTimestamp
{
   int seconds; /* microseconds */
   short year;
   signed char month;
   signed char day;
   signed char hour;
   signed char minute;
} ODPPTeraTimestamp;

#define ODPPTeraMaskInitSQLCALatin(sqlcaaddr, sqlstate, functionName, specificName, errorMessage) \
{ \
   ODPPTeraMaskSQLCA* pSQLCA = sqlcaaddr; \
   ODPPTeraMaskInitSQLCALatinFunc(pSQLCA, sqlstate, functionName, specificName, errorMessage); \
   pSQLCA->pFNC_GetStructuredAttributeCount = &FNC_GetStructuredAttributeCount; \
   pSQLCA->pFNC_GetStructuredAttributeByNdx = &FNC_GetStructuredAttributeByNdx; \
   pSQLCA->pFNC_SetStructuredAttributeByNdx = &FNC_SetStructuredAttributeByNdx; \
   pSQLCA->pFNC_GetStructuredAttributeInfo = &FNC_GetStructuredAttributeInfo; \
   pSQLCA->pFNC_GetPhase = &FNC_GetPhase; \
   pSQLCA->pFNC_TblAllocCtx = &FNC_TblAllocCtx; \
   pSQLCA->pFNC_TblGetCtx = &FNC_TblGetCtx; \
   pSQLCA->pFNC_TblGetColDef = &FNC_TblGetColDef; \
   pSQLCA->pFNC_malloc = &FNC_malloc; \
   pSQLCA->pFNC_free = &FNC_free; \
   pSQLCA->pFNC_Trace_String = &FNC_Trace_String; \
   pSQLCA->pFNC_Trace_Write_DL = &FNC_Trace_Write_DL; \
}

#define ODPPTeraMaskInitSQLCAUTF16(sqlcaaddr, sqlstate, functionNameW, specificNameW, errorMessageW) \
{ \
   ODPPTeraMaskSQLCA* pSQLCA = sqlcaaddr; \
   ODPPTeraMaskInitSQLCAUTF16Func(pSQLCA, sqlstate, functionNameW, specificNameW, errorMessageW); \
   pSQLCA->pFNC_GetStructuredAttributeCount = &FNC_GetStructuredAttributeCount; \
   pSQLCA->pFNC_GetStructuredAttributeByNdx = &FNC_GetStructuredAttributeByNdx; \
   pSQLCA->pFNC_SetStructuredAttributeByNdx = &FNC_SetStructuredAttributeByNdx; \
   pSQLCA->pFNC_GetStructuredAttributeInfo = &FNC_GetStructuredAttributeInfo; \
   pSQLCA->pFNC_GetPhase = &FNC_GetPhase; \
   pSQLCA->pFNC_TblAllocCtx = &FNC_TblAllocCtx; \
   pSQLCA->pFNC_TblGetCtx = &FNC_TblGetCtx; \
   pSQLCA->pFNC_TblGetColDef = &FNC_TblGetColDef; \
   pSQLCA->pFNC_malloc = &FNC_malloc; \
   pSQLCA->pFNC_free = &FNC_free; \
   pSQLCA->pFNC_Trace_String = &FNC_Trace_String; \
   pSQLCA->pFNC_Trace_Write_DL = &FNC_Trace_Write_DL; \
}

EXPORT
void ODPPTeraMaskInitSQLCALatinFunc(ODPPTeraMaskSQLCA *pSQLCA, char *sqlstate, unsigned char *functionName, unsigned char *specificName, unsigned char *errorMessage);

EXPORT
void ODPPTeraMaskInitSQLCAUTF16Func(ODPPTeraMaskSQLCA *pSQLCA, char *sqlstate, unsigned short *functionNameW, unsigned short *specificNameW, unsigned short *errorMessageW);

EXPORT
int ODPPTeraMaskStrLatin(unsigned char* szProviderStr, unsigned char* szInValue, int iInNullInd, unsigned char* szOutValue, int* piOutNullInd, size_t OutLen, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskStrLatinUDT(unsigned char* szProviderStr, UDT_HANDLE InValue, int iInNullInd, unsigned char* szOutValue, int* piOutNullInd, size_t OutLen, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskStrUTF16(unsigned short* szProviderStr, unsigned short* szInValue, int iInNullInd, unsigned short* szOutValue, int* piOutNullInd, size_t OutLen, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskStrUTF16UDT(unsigned short* szProviderStr, UDT_HANDLE InValue, int iInNullInd, unsigned short* szOutValue, int* piOutNullInd, size_t OutLen, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskInt64(char *szProviderStr, long long inValue, int iInNullInd, long long* pOutValue, int* piOutNullInd, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskInt64Date(char *szProviderStr, DATE inValue, int iInNullInd, long long* pOutValue, int* piOutNullInd, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskInt64Double(char *szProviderStr, double inValue, int iInNullInd, long long* pOutValue, int* piOutNullInd, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskInt64StrUTF16(unsigned short* szProviderStr, unsigned short* szInValue, int iInNullInd, long long* pOutValue, int* piOutNullInd, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskInt64Time(char *szProviderStr, ODPPTeraTime* pInValue, int iInNullInd, long long* pOutValue, int* piOutNullInd, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskInt64Timestamp(char *szProviderStr, ODPPTeraTimestamp* pInValue, int iInNullInd, long long* pOutValue, int* piOutNullInd, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskInt64UDT(char *szProviderStr, UDT_HANDLE InValue, int iInNullInd, long long* pOutValue, int* piOutNullInd, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskDouble(char *szProviderStr, double inValue, int iInNullInd, double* pOutValue, int* piOutNullInd, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskDec8(char *szProviderStr, long long inValue, int iInNullInd, int iPrecision, short sScale, long long* pOutValue, int* piOutNullInd, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskDate(char *szProviderStr, DATE inValue, int iInNullInd, DATE* pOutValue, int* piOutNullInd, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskTime(char* szProviderStr, ODPPTeraTime* pInValue, int iInNullInd, ODPPTeraTime* pOutValue, int* piOutNullInd, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskTimestamp(char* szProviderStr, ODPPTeraTimestamp* pInValue, int iInNullInd, ODPPTeraTimestamp* pOutValue, int* piOutNullInd, ODPPTeraMaskSQLCA *pSQLCA);

EXPORT
int ODPPTeraMaskTable(unsigned short* szProviderStr,
                      int iProviderStrNullInd,
                      UDT_HANDLE KeyList,
                      int iKeyListNullInd,
                      UDT_HANDLE InValue,
                      int iInNullInd,
                      void* pOutValue[],
                      int *piOutNullInd[],
                      ODPPTeraMaskSQLCA *pSQLCA);
